@@ -4,13 +4,17 @@
 :copyright: 2008-2010 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
 :contact: http://www.logilab.fr/ -- mailto:contact@logilab.fr
 """
+
+from __future__ import with_statement
+
 __docformat__ = "restructuredtext en"
 
-from os.path import dirname, join, abspath
+from os.path import join
 
 from logilab.common.decorators import cached
 
 from cubicweb import Unauthorized, ValidationError
+from cubicweb.server.session import security_enabled
 from cubicweb.devtools import (BaseApptestConfiguration, TestServerConfiguration,
                                ApptestConfiguration, init_test_database)
 from cubicweb.devtools.testlib import CubicWebTC
@@ -54,7 +58,8 @@ class TrackerTCMixIn(object):
         """
         pname = unicode(pname)
         plabel = plabel and unicode(plabel) or unicode(group)
-        peid = self.session.unsafe_execute(
+        with security_enabled(self.session, False, False):
+            peid = self.execute(
             'INSERT CWPermission X: X name %%(pname)s, X label %%(plabel)s,'
             'X require_group G, '
             'P require_permission X '
@@ -78,30 +83,41 @@ class TrackerBaseTC(TrackerTCMixIn, CubicWebTC):
 
 class SecurityTC(TrackerTCMixIn, CubicWebTC):
     repo_config = BaseApptestConfiguration('data')
-    _initialized = False
     def setUp(self):
+        """
+            User created:
+                - stduser
+                - staffuser
+                - prj1developer
+                - prj1client
+                - prj2developer
+                - prj2client
+        """
         CubicWebTC.setUp(self)
         self.__cnxs = {'admin': self.cnx}
-        # trick to avoid costly initialization for each test
-        if not SecurityTC._initialized:
-            # implicitly test manager can add some entities
-            self.__class__.cubicweb = self.execute(*create_project_rql("cubicweb")).get_entity(0, 0)
-            self.execute(*create_project_rql("projet2"))
-            self.execute('INSERT CWGroup X: X name "cubicwebdevelopers"')
-            self.execute('INSERT CWGroup X: X name "cubicwebclients"')
-            self.grant_permission('cubicweb', 'cubicwebdevelopers', u'developer')
-            self.grant_permission('cubicweb', 'cubicwebclients', u'client')
-            self.commit()
-            self.create_user('stduser')
-            self.create_user('staffuser', groups=('users', 'staff',))
-            self.create_user('prj1developer', groups=('users', 'cubicwebdevelopers',))
-            self.create_user('prj1client', groups=('users', 'cubicwebclients'))
-            self.maxeid = self.execute('Any MAX(X)')[0][0]
-            #SecurityTC._initialized = True
-            cachedperms = self.execute('Any UL, PN WHERE U has_group_permission P, U login UL, P label PN')
-            self.assertEquals(len(cachedperms), 2)
-            self.assertEquals(dict(cachedperms),
-                              {'prj1developer': 'cubicwebdevelopers', 'prj1client': 'cubicwebclients'})
+        self.cubicweb = self.execute(*create_project_rql("cubicweb")).get_entity(0, 0)
+        self.project2 = self.execute(*create_project_rql("project2")).get_entity(0,0)
+        for prj in self.cubicweb, self.project2:
+            name = prj.name
+            self.execute('INSERT CWGroup X: X name "%sdevelopers"' %name)
+            self.execute('INSERT CWGroup X: X name "%sclients"' % name)
+            self.grant_permission(name, '%sdevelopers' % name, u'developer')
+            self.grant_permission(name, '%sclients' % name, u'client')
+        self.commit()
+        self.create_user('stduser')
+        self.create_user('staffuser', groups=('users', 'staff',))
+        self.create_user('prj1developer', groups=('users', 'cubicwebdevelopers',))
+        self.create_user('prj1client', groups=('users', 'cubicwebclients'))
+        self.create_user('prj2developer', groups=('users', 'project2developers',))
+        self.create_user('prj2client', groups=('users', 'project2clients'))
+        self.maxeid = self.execute('Any MAX(X)')[0][0]
+        cachedperms = self.execute('Any UL, PN WHERE U has_group_permission P, U login UL, P label PN')
+        self.assertEquals(len(cachedperms), 4)
+        self.assertDictEquals(dict(cachedperms),
+                          {'prj1developer': 'cubicwebdevelopers',
+                           'prj1client':    'cubicwebclients',
+                           'prj2developer': 'project2developers',
+                           'prj2client':    'project2clients'})
 
     def mylogin(self, user):
         if not user in self.__cnxs:
@@ -115,7 +131,7 @@ class SecurityTC(TrackerTCMixIn, CubicWebTC):
             # if the user can't see entity x, Unauthorized is raised, else if he
             # can't pass the transition, Validation is raised
             self.assertRaises((Unauthorized, ValidationError),
-                              entity.fire_transition, trname)
+                              entity.cw_adapt_to('IWorkflowable').fire_transition, trname)
         finally:
             cnx.rollback()
 
@@ -123,7 +139,7 @@ class SecurityTC(TrackerTCMixIn, CubicWebTC):
         cnx = self.mylogin(user)
         try:
             entity = cnx.request().entity_from_eid(x)
-            entity.fire_transition(trname)
+            entity.cw_adapt_to('IWorkflowable').fire_transition(trname)
             cnx.commit()
         except:
             cnx.rollback()
@@ -139,14 +155,14 @@ class ThreeSourcesConfiguration(ApptestConfiguration):
                          'pyro-ns-id': 'extern',
                          'cubicweb-user': 'admin',
                          'cubicweb-password': 'gingkow',
-                         'mapping-file': abspath(join(dirname(__file__), 'test', 'data', 'extern_mapping.py'))
+                         'mapping-file': 'extern_mapping.py'
                          #'base-url': '',
                          }
         res['extern2'] = {'adapter':    'pyrorql',
                          'pyro-ns-id': 'extern2',
                          'cubicweb-user': 'admin',
                          'cubicweb-password': 'gingkow',
-                         'mapping-file': abspath(join(dirname(__file__), 'test', 'data', 'extern_mapping.py'))
+                         'mapping-file': 'extern_mapping.py'
                          #'base-url': '',
                          }
         if self._enabled_sources is None:
@@ -224,7 +240,6 @@ class BaseMSTC(CubicWebTC):
         # concurrent access to the database
         self.execute('Any P, V WHERE V? version_of P')
         self.execute('State X')
-        self.maxeid = self.execute('Any MAX(X)')[0][0]
         self.ip1eid = self.execute('INSERT Project X: X name "Intern project 1"')[0][0]
         self.ip1v1eid = self.execute('INSERT Version V: V num "0.1.0", V version_of X WHERE X name "Intern project 1"')[0][0]
         self.ip1t1eid = self.execute('INSERT Ticket X: X title "ticket project 1", X concerns P, X done_in V WHERE P eid %(p)s, V eid %(v)s',
@@ -240,10 +255,10 @@ class BaseMSTC(CubicWebTC):
         return self.repo.extid2eid(self.extern, str(extid), etype, self.session)
 
     def fire_transition(self, eid, trname):
-        e = self.execute('Any X WHERE X eid %(x)s', {'x': eid}, 'x').get_entity(0, 0)
-        e.fire_transition(trname)
+        e = self.execute('Any X WHERE X eid %(x)s', {'x': eid}).get_entity(0, 0)
+        e.cw_adapt_to('IWorkflowable').fire_transition(trname)
 
     def change_state(self, eid, stname):
-        e = self.execute('Any X WHERE X eid %(x)s', {'x': eid}, 'x').get_entity(0, 0)
-        e.change_state(stname)
+        e = self.execute('Any X WHERE X eid %(x)s', {'x': eid}).get_entity(0, 0)
+        e.cw_adapt_to('IWorkflowable').change_state(stname)
 
