@@ -1,7 +1,7 @@
 """Restructured text view to export content of a tracker instance
 
 :organization: Logilab
-:copyright: 2007-2009 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
+:copyright: 2007-2010 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
 :contact: http://www.logilab.fr/ -- mailto:contact@logilab.fr
 """
 __docformat__ = "restructuredtext en"
@@ -9,7 +9,7 @@ __docformat__ = "restructuredtext en"
 from logilab.common.textutils import normalize_text
 
 from cubicweb.view import EntityView
-from cubicweb.selectors import (implements, match_search_state, one_line_rset,
+from cubicweb.selectors import (is_instance, match_search_state, one_line_rset,
                                 multi_lines_rset)
 from cubicweb.entity import _marker
 from cubicweb.web import action
@@ -34,7 +34,7 @@ class ReSTItemView(EntityView):
     def field(self, entity, attr, value=_marker, **kwargs):
         if value is _marker:
             if attr == 'state':
-                value = entity.printable_state
+                value = entity.cw_adapt_to('IWorkflowable').printable_state
             else:
                 value = entity.printable_value(attr, value, format='text/plain',
                                                **kwargs)
@@ -69,7 +69,7 @@ class ReSTItemView(EntityView):
 
 class ProjectDocumentItemView(ReSTItemView):
     """ReST view for Project entities"""
-    __select__ = implements('Project')
+    __select__ = is_instance('Project')
 
     title_underline = '='
     fields = ['creation_date']
@@ -83,7 +83,7 @@ class ProjectDocumentItemView(ReSTItemView):
 
 class VersionDocumentItemView(ReSTItemView):
     """ReST view for Version entities"""
-    __select__ = implements('Version')
+    __select__ = is_instance('Version')
 
     title_underline = '-'
 
@@ -93,7 +93,7 @@ class VersionDocumentItemView(ReSTItemView):
 
     def render_attributes(self, entity):
         self.paragraph_field(entity, 'description')
-        if entity.state == 'published':
+        if entity.cw_adapt_to('IWorkflowable').state == 'published':
             self.field(entity, 'publication_date')
         else:
             self.field(entity, 'prevision_date')
@@ -105,7 +105,7 @@ class VersionDocumentItemView(ReSTItemView):
 
 class TicketDocumentItemView(ReSTItemView):
     """ReST view for Ticket entities"""
-    __select__ = implements('Ticket')
+    __select__ = is_instance('Ticket')
 
     title_underline = '~'
     fields = ['type', 'state']
@@ -119,7 +119,7 @@ class TicketDocumentItemView(ReSTItemView):
 
 class DocumentView(EntityView):
     __regid__ = 'document'
-    __select__ = implements('Project', 'Version', 'Ticket')
+    __select__ = is_instance('Project', 'Version', 'Ticket')
 
     title = _('document')
     templatable = False
@@ -144,7 +144,7 @@ class DocumentView(EntityView):
 class ProjectVersionExportAction(action.Action):
     __regid__ = 'pvrestexport'
     __select__ = (match_search_state('normal') & one_line_rset() &
-                  implements('Project', 'Version'))
+                  is_instance('Project', 'Version'))
 
     title = _('ReST export')
     order = 410
@@ -157,7 +157,7 @@ class ProjectVersionExportAction(action.Action):
 class TicketRESTExportAction(action.Action):
     __regid__ = 'ticketrestexport'
     __select__ = (match_search_state('normal') & multi_lines_rset() &
-                  implements('Ticket'))
+                  is_instance('Ticket'))
     title = _('ReST export')
 
     def url(self):
