@@ -1,7 +1,7 @@
 """views for Ticket entities
 
 :organization: Logilab
-:copyright: 2006-2009 LOGILAB S.A. (Paris, FRANCE), license is LGPL.
+:copyright: 2006-2010 LOGILAB S.A. (Paris, FRANCE), license is LGPL.
 :contact: http://www.logilab.fr/ -- mailto:contact@logilab.fr
 """
 __docformat__ = "restructuredtext en"
@@ -9,13 +9,13 @@ _ = unicode
 
 from logilab.mtconverter import xml_escape
 
-from cubicweb.view import EntityView
+from cubicweb.view import EntityView, EntityAdapter
 from cubicweb.schema import display_name
-from cubicweb.selectors import (objectify_selector, implements, multi_lines_rset,
+from cubicweb.selectors import (objectify_selector, is_instance, multi_lines_rset,
                                 score_entity)
 from cubicweb import tags, uilib
 from cubicweb.web import uicfg, component, action
-from cubicweb.web.views import primary, baseviews
+from cubicweb.web.views import primary, baseviews, ibreadcrumbs
 
 _pvs = uicfg.primaryview_section
 
@@ -40,14 +40,14 @@ _pvdc.tag_attribute(('Ticket', 'description'), {'showlabel': False})
 class TicketPrimaryView(primary.PrimaryView):
     """primary view for tickets
     """
-    __select__ = primary.PrimaryView.__select__ & implements('Ticket')
+    __select__ = primary.PrimaryView.__select__ & is_instance('Ticket')
 
     def render_entity_title(self, entity):
         self.w(u'<h1><span class="%s %s">%s %s</span><span class="state"> [%s]</span></h1>'
                % (entity.priority, entity.type,
                   xml_escape(entity.project.name),
                   xml_escape(entity.dc_title()),
-                  xml_escape(self._cw._(entity.state))))
+                  xml_escape(self._cw._(entity.cw_adapt_to('IWorkflowable').state))))
 
 
 # pluggable sections ###########################################################
@@ -55,7 +55,7 @@ class TicketPrimaryView(primary.PrimaryView):
 class TicketIdenticalToVComponent(component.RelatedObjectsVComponent):
     """display identical tickets"""
     __regid__ = 'tickectidentical'
-    __select__ = component.RelatedObjectsVComponent.__select__ & implements('Ticket')
+    __select__ = component.RelatedObjectsVComponent.__select__ & is_instance('Ticket')
 
     rtype = 'identical_to'
     target = 'object'
@@ -73,13 +73,13 @@ class TicketOneLineView(baseviews.OneLineView):
     call text view to get link's label, set entity's description on the
     link and display entity's status
     """
-    __select__ = implements('Ticket')
+    __select__ = is_instance('Ticket')
 
     def cell_call(self, row, col):
         self.wview('incontext', self.cw_rset, row=row)
         entity = self.cw_rset.get_entity(row, col)
         if entity.in_state:
-            self.w(u'&nbsp;[%s]' % xml_escape(self._cw._(entity.state)))
+            self.w(u'&nbsp;[%s]' % xml_escape(self._cw._(entity.cw_adapt_to('IWorkflowable').state)))
 
 
 class TicketInContextView(baseviews.OneLineView):
@@ -89,7 +89,7 @@ class TicketInContextView(baseviews.OneLineView):
     link and display entity's status
     """
     __regid__ = 'incontext'
-    __select__ = implements('Ticket')
+    __select__ = is_instance('Ticket')
 
     def cell_call(self, row, col):
         self._cw.add_css('cubes.tracker.css')
@@ -101,7 +101,7 @@ class TicketInContextView(baseviews.OneLineView):
 
 class StatusSheetTicketView(EntityView):
     __regid__ = 'instatussheet'
-    __select__ = implements('Ticket')
+    __select__ = is_instance('Ticket')
 
     def cell_call(self, row, col):
         entity = self.cw_rset.get_entity(row, col)
@@ -110,6 +110,42 @@ class StatusSheetTicketView(EntityView):
                         title=xml_escape(entity.title),
                         style='display: inline'))
 
+# adapters #####################################################################
+
+class TicketIBreadCrumbsAdapter(ibreadcrumbs.IBreadCrumbsAdapter):
+    __select__ = is_instance('Ticket')
+
+    def parent_entity(self):
+        parents = self.entity.done_in or self.entity.concerns
+        if parents:
+            return parents[0]
+
+
+class IPrevNextAdapter(EntityAdapter):
+    """interface for entities which can be linked to a previous and/or next
+    entity
+    """
+    __regid__ = 'IPrevNext'
+    __select__ = is_instance('Ticket')
+
+    def previous_entity(self):
+        ticket = self.entity
+        rql = ('Any X,T ORDERBY X DESC LIMIT 1 '
+               'WHERE X is Ticket, X concerns P, X title T, P eid %(p)s, '
+               'X eid < %(x)s')
+        rset = self._cw.execute(rql, {'p': ticket.project.eid, 'x': ticket.eid})
+        if rset:
+            return rset.get_entity(0, 0)
+
+    def next_entity(self):
+        ticket = self.entity
+        rql = ('Any X,T ORDERBY X ASC LIMIT 1 '
+               'WHERE X is Ticket, X concerns P, X title T, P eid %(p)s, '
+               'X eid > %(x)s')
+        rset = self._cw.execute(rql, {'p': ticket.project.eid, 'x': ticket.eid})
+        if rset:
+            return rset.get_entity(0, 0)
+
 
 # actions ######################################################################
 
@@ -163,7 +199,7 @@ def ticket_has_next_version(cls, req, rset, row=None, col=0, **kwargs):
 
 
 class TicketAction(action.Action):
-    __select__ = action.Action.__select__ & implements('Ticket')
+    __select__ = action.Action.__select__ & is_instance('Ticket')
     # use "mainactions" category to appears in table filter's actions menu
     category = 'mainactions'
 
@@ -171,7 +207,7 @@ class TicketAction(action.Action):
 class TicketMoveToNextVersionActions(TicketAction):
     __regid__ = 'movetonext'
     __select__ = (TicketAction.__select__
-                  & score_entity(lambda x: x.state in x.OPEN_STATES)
+                  & score_entity(lambda x: x.cw_adapt_to('IWorkflowable').state in x.OPEN_STATES)
                   & ticket_has_next_version())
 
     submenu = _('move to version')
