@@ -7,14 +7,17 @@
 __docformat__ = "restructuredtext en"
 _ = unicode
 
+from datetime import date
+
 from logilab.mtconverter import xml_escape
 
-from cubicweb.selectors import (objectify_selector, implements, score_entity,
+from cubicweb.selectors import (objectify_selector, is_instance, score_entity,
                                 match_search_state, one_line_rset)
 from cubicweb.view import StartupView
 from cubicweb import tags
 from cubicweb.web import uicfg, component, action
-from cubicweb.web.views import primary, baseviews, iprogress, xmlrss
+from cubicweb.web.views import (primary, baseviews, xmlrss, navigation,
+                                iprogress, ibreadcrumbs)
 
 _pvs = uicfg.primaryview_section
 
@@ -30,14 +33,14 @@ for attr in ('num', 'description', 'starting_date', 'prevision_date'):
 
 
 class VersionPrimaryView(primary.PrimaryView):
-    __select__ = implements('Version')
+    __select__ = is_instance('Version')
 
     def render_entity_title(self, entity):
         self._cw.add_css('cubes.tracker.css')
         self.w(u'<h1>') #% entity.progress_class())
         self.w('%s %s <span class="state">[%s]</span>' % (
             xml_escape(entity.project.name), xml_escape(entity.num),
-            xml_escape(self._cw._(entity.state))))
+            xml_escape(self._cw._(entity.cw_adapt_to('IWorkflowable').state))))
         self.w(u'</h1>\n')
 
     def render_entity_attributes(self, entity):
@@ -73,12 +76,34 @@ class VersionPrimaryView(primary.PrimaryView):
                            entity.defects_rql_nb_displayed_cols, 'defects')
 
 
+class VersionIBreadCrumbsAdapter(ibreadcrumbs.IBreadCrumbsAdapter):
+    __select__ = is_instance('Version')
+
+    def parent_entity(self):
+        return self.entity.project
+
+
+class VersionIPrevNextAdapter(navigation.IPrevNextAdapter):
+    __select__ = is_instance('Version')
+
+    def previous_entity(self):
+        found = False
+        for version in self.entity.project.reverse_version_of:
+            if found:
+                return version
+            if version is self:
+                found = True
+
+    def next_entity(self):
+        return self.entity.next_version(states=None)
+
+
 # pluggable sections ###########################################################
 
 class VersionInfoVComponent(component.EntityVComponent):
     """display version information table in the context of the project"""
     __regid__ = 'versioninfo'
-    __select__ = component.EntityVComponent.__select__ & implements('Version')
+    __select__ = component.EntityVComponent.__select__ & is_instance('Version')
     context = 'navcontenttop'
     order = 10
 
@@ -100,17 +125,17 @@ class VersionInfoVComponent(component.EntityVComponent):
 # secondary views ##############################################################
 
 class VersionTextView(baseviews.TextView):
-    __select__ = implements('Version')
+    __select__ = is_instance('Version')
 
     def cell_call(self, row, col):
         entity = self.cw_rset.get_entity(row, col)
         self.w(entity.num)
         if entity.in_state:
-            self.w(u' [%s]' % self._cw._(entity.state))
+            self.w(u' [%s]' % self._cw._(entity.cw_adapt_to('IWorkflowable').state))
 
 
 class VersionIncontextView(baseviews.InContextView):
-    __select__ = implements('Version')
+    __select__ = is_instance('Version')
 
     def cell_call(self, row, col):
         entity = self.cw_rset.get_entity(row, col)
@@ -118,7 +143,7 @@ class VersionIncontextView(baseviews.InContextView):
 
 
 class VersionOutOfContextView(baseviews.OutOfContextView):
-    __select__ = implements('Version')
+    __select__ = is_instance('Version')
 
     def cell_call(self, row, col):
         entity = self.cw_rset.get_entity(row, col)
@@ -129,14 +154,14 @@ class VersionOutOfContextView(baseviews.OutOfContextView):
         self.w(u'<a href="%s">' % xml_escape(entity.absolute_url()))
         self.wdata(entity.num)
         if entity.in_state:
-            self.wdata(u' [%s]' % self._cw._(entity.state))
+            self.wdata(u' [%s]' % self._cw._(entity.cw_adapt_to('IWorkflowable').state))
         self.w(u'</a>')
 
 
 # other views ##################################################################
 
 class VersionProgressTableView(iprogress.ProgressTableView):
-    __select__ = implements('Version')
+    __select__ = is_instance('Version')
 
     title = _('version progression')
 
@@ -158,9 +183,10 @@ class VersionProgressTableView(iprogress.ProgressTableView):
 
     def build_planned_delivery_cell(self, entity):
         """``initial_prevision_date`` column cell renderer"""
-        if entity.finished():
-            return self._cw.format_date(entity.completion_date())
-        return self._cw.format_date(entity.initial_prevision_date())
+        imilestone = entity.cw_adapt_to('IMileStone')
+        if imilestone.finished():
+            return self._cw.format_date(imilestone.completion_date())
+        return self._cw.format_date(imilestone.initial_prevision_date())
 
 
 class VersionsInfoView(StartupView):
@@ -182,7 +208,7 @@ class VersionsInfoView(StartupView):
 
 
 class RssItemVersionView(xmlrss.RSSItemView):
-    __select__ = implements('Version')
+    __select__ = is_instance('Version')
 
     def cell_call(self, row, col):
         entity = self.cw_rset.complete_entity(row, col)
@@ -205,8 +231,8 @@ class RssItemVersionView(xmlrss.RSSItemView):
 
 class VersionAddTicketAction(action.LinkToEntityAction):
     __regid__ = 'addticket'
-    __select__ = (action.LinkToEntityAction.__select__ & implements('Version')
-                  & score_entity(lambda x: x.state not in x.FINISHED_STATES))
+    __select__ = (action.LinkToEntityAction.__select__ & is_instance('Version')
+                  & score_entity(lambda x: not x.cw_adapt_to('IProgress').finished()))
 
     title = _('add Ticket done_in Version object')
     target_etype = 'Ticket'
@@ -221,8 +247,8 @@ class VersionAddTicketAction(action.LinkToEntityAction):
 
 class VersionSubmitBugAction(VersionAddTicketAction):
     __regid__ = 'submitbug'
-    __select__ = (action.LinkToEntityAction.__select__ & implements('Version')
-                  & score_entity(lambda x: x.state in x.PUBLISHED_STATES))
+    __select__ = (action.LinkToEntityAction.__select__ & is_instance('Version')
+                  & score_entity(lambda x: x.cw_adapt_to('IWorkflowable').state in x.PUBLISHED_STATES))
 
     title = _('add Ticket appeared_in Version object')
     rtype = 'appeared_in'
@@ -236,3 +262,31 @@ _abaa.tag_object_of(('*', 'appeared_in', 'Version'), False)
 # register messages generated for the form title until catalog generation is fixed
 _('creating Ticket (Ticket done_in Version %(linkto)s)')
 _('creating Ticket (Ticket appeared_in Version %(linkto)s)')
+
+from cubicweb.view import EntityAdapter
+
+class VersionICalendarableAdapter(EntityAdapter):
+    __regid__ = 'ICalendarable'
+    __select__ = is_instance('Version')
+
+    @property
+    def start(self):
+        return self.entity.start_date() or self.entity.starting_date or date.today()
+
+    @property
+    def stop(self):
+        return self.entity.stop_date() or self.entity.prevision_date or date.today()
+
+
+class VersionICalendarViewsAdapter(EntityAdapter):
+    __regid__ = 'ICalendarViews'
+    __select__ = is_instance('Version')
+
+    def matching_dates(self, begin, end):
+        """return prevision or publication date according to state"""
+        if self.entity.in_state[0].name in self.entity.PUBLISHED_STATES:
+            if self.entity.publication_date:
+                return [self.entity.publication_date]
+        elif self.entity.prevision_date:
+            return [self.entity.prevision_date]
+        return []
